part of tags;

const Clipboard clipboard = const Clipboard();

class ClipboardItem {
  final source;
  final String type;  
  String data;
  var context;
  ClipboardItem(this.source, this.type);
}

class Clipboard {
  
  static final List<ClipboardItem> _currentItems = new List<ClipboardItem>();
  static final Map<String, Function> _copy = new Map<String, Function>();
  static final Map<String, Function> _canPaste = new Map<String, Function>();
  static final Map<String, Function> _paste = new Map<String, Function>();
  static StreamSubscription _documentOnCopyListener;
  
  const Clipboard();
  
  void register(String type, copy(ClipboardItem item), 
        bool canPaste(ClipboardItem item, target), 
        paste(ClipboardItem item, target)) {    
    _copy[type] = (item) => new Future.sync(() => copy(item));
    _canPaste[type] = canPaste;
    _paste[type] = (item, target) => new Future.sync(() => paste(item, target));
    _listenDocument();
  }
  
  void clear() => _currentItems.clear();
  
  Future copy(String type, source) {
    if (!_copy.containsKey(type)) return new Future.value(false);    
    final ClipboardItem item = new ClipboardItem(source, type);
    return _copy[type](item).then((_) {
      final done = dom.document.onCopy.first.then((ev) {
        ev.preventDefault();
        ev.clipboardData.items.clear();
        _currentItems.clear();
        ev.clipboardData.items.addData(item.data, type);
        _currentItems.add(item);
      });
      dom.document.execCommand("copy", false, null);
      return done;      
    });
  }
  
  bool canPaste(target) {
    // TODO: support multiple items on the clipboard
    if (_currentItems.length != 1) return false;
    final ClipboardItem currentItem = _currentItems[0];
    return _canPaste[currentItem.type](currentItem, target);
  }
  
  Future paste(target) {
    if (!canPaste(target)) return new Future.value();
    final done = dom.document.onPaste.first.then((ev) {
      ev.preventDefault();
      final item = ev.clipboardData.items[0];
      final ClipboardItem currentItem = _currentItems[0];
      if (item.type != currentItem.type) {
        return new Future.value();
      }          
      final String data = ev.clipboardData.getData(item.type);
      return _paste[item.type](currentItem, target);
    });
    dom.document.execCommand("paste", false, null);
    return done;
  }
  
  void _listenDocument() {
    if (_documentOnCopyListener == null) {
      _documentOnCopyListener = dom.document.onCopy.listen((ev) {
        // When another entity copies to the system clipboard clear our items.
        if (ev.clipboardData.items.length == 0 ||
            !_copy.keys.contains(ev.clipboardData.items[0].type)) {
          _currentItems.clear();
        }
      });
    }
  } 
}
