part of tags;

class Command {
  
  static final _canExecuteAlways = () => true;
  static final _canExecuteNever = () => false;
  
  /// A command that always returns `false` for [canExecute].
  static final Command noop = new Command._();
  
  final Function _execute;
  final Function _canExecute;
  /// Whether or not this command may be called to execute at the present time.
  bool get canExecute => !_isExecuting && _canExecute();
  
  bool _isExecuting = false;
  /// Whether or not this command is executing at the present time.
  bool get isExecuting => _isExecuting;
  
  /// Creates a new command with the given [canExecute] and [execute] functions.
  /// 
  /// The given [execute] function may return any value including a future if it
  /// performs an asynchronous computation.
  Command(bool canExecute(), execute()) 
  : _canExecute = canExecute
  , _execute = (() => new Future.sync(execute));

  /// Creates a new command with the given [execute] function that can always
  /// execute.
  Command.always(execute()) : this(_canExecuteAlways, execute);
  
  Command._() : this(_canExecuteNever, null);
  
  /// Executes this command if [canExecute] is `true`.
  /// 
  /// Returns a future that completes with the result of the computation.
  Future call() {
    if (!canExecute) {
      return new Future.value();
    }
    _isExecuting = true;
    return _execute().whenComplete(() => _isExecuting = false);
  }
}
