library tags;

import 'dart:async';
import 'dart:collection';

part 'model/builder.dart';
part 'model/clipboard.dart';
part 'model/command.dart';
part 'model/editor.dart';
part 'model/key_binding.dart';
part 'model/resources.dart';
part 'model/routed_command.dart';
part 'model/selections.dart';
part 'model/theme.dart';
part 'model/themeable.dart';
part 'model/toggleable.dart';
part 'model/tree.dart';

Dom _dom;
/// The current Document Object Model used by this library.
/// 
/// The [Tag] builder functions delegate to this dom to create elements and 
/// nodes.  The dom is decoupled to allow for building trees of tags without a 
/// dependency on `dart:html`. 
/// 
/// This must be set by the user of this library prior to making any other
/// calls to this library.  This will typically be set in a `main` function or
/// other bootstrapping logic.  For example:
/// 
///     import 'package:tags/tags.dart' as tags;
///     import 'package:tags/html.dart';
///
///     main() {
///       tags.dom = HTML;
///       ...
///     }
Dom 
  get dom => _dom;
  set dom(Dom dom) {
    _dom = dom;
    _dom.registerElements();
  }
  
abstract class Dom {  
  const Dom();  
  get document;  
  void registerElements();  
  createElement(String tag, [String typeExtension]);  
  createSvgElement(String tag);
}

final Tag a         = createTag((_) => dom.createElement('a'));
final Tag area      = createTag((_) => dom.createElement('area'));
final Tag article   = createTag((_) => dom.createElement('article'));
final Tag aside     = createTag((_) => dom.createElement('aside'));
final Tag audio     = createTag((_) => dom.createElement('audio'));
final Tag b         = createTag((_) => dom.createElement('b'));
final Tag base      = createTag((_) => dom.createElement('base'));
final Tag body      = createTag((_) => dom.createElement('body'));
final Tag br        = createTag((_) => dom.createElement('br'));
final Tag button    = createTag((_) => dom.createElement('button'));
final Tag canvas    = createTag((_) => dom.createElement('canvas'));
final Tag caption   = createTag((_) => dom.createElement('caption'));
final Tag code      = createTag((_) => dom.createElement('code'));
final Tag col       = createTag((_) => dom.createElement('col'));
final Tag content   = createTag((_) => dom.createElement('content'));
final Tag datalist  = createTag((_) => dom.createElement('datalist'));
final Tag dd        = createTag((_) => dom.createElement('dd'));
final Tag del       = createTag((_) => dom.createElement('del'));
final Tag details   = createTag((_) => dom.createElement('details'));
final Tag div       = createTag((_) => dom.createElement('div'));
final Tag dl        = createTag((_) => dom.createElement('dl'));
final Tag dt        = createTag((_) => dom.createElement('dt'));
final Tag editableLabel = createTag((_) => 
    dom.createElement('div', 'editable-label'));
final Tag embed     = createTag((_) => dom.createElement('embed'));
final Tag fieldset  = createTag((_) => dom.createElement('fieldset'));
final Tag footer    = createTag((_) => dom.createElement('footer'));
final Tag form      = createTag((_) => dom.createElement('form'));
final Tag h1        = createTag((_) => dom.createElement('h1'));
final Tag h2        = createTag((_) => dom.createElement('h2'));
final Tag h3        = createTag((_) => dom.createElement('h3'));
final Tag h4        = createTag((_) => dom.createElement('h4'));
final Tag h5        = createTag((_) => dom.createElement('h5'));
final Tag h6        = createTag((_) => dom.createElement('h6'));
final Tag head      = createTag((_) => dom.createElement('head'));
final Tag header    = createTag((_) => dom.createElement('header'));
final Tag hr        = createTag((_) => dom.createElement('hr'));
final Tag html      = createTag((_) => dom.createElement('html'));
final Tag i         = createTag((_) => dom.createElement('i'));
final Tag iframe    = createTag((_) => dom.createElement('iframe'));
final Tag img       = createTag((_) => dom.createElement('img'));
final Tag input     = createTag((_) => dom.createElement('input'));
final Tag keygen    = createTag((_) => dom.createElement('keygen'));
final Tag label     = createTag((_) => dom.createElement('label'));
final Tag legend    = createTag((_) => dom.createElement('legend'));
final Tag li        = createTag((_) => dom.createElement('li'));
final Tag link      = createTag((_) => dom.createElement('link'));
final Tag map       = createTag((_) => dom.createElement('map'));
final Tag menu      = createTag((_) => dom.createElement('menu'));
final Tag meter     = createTag((_) => dom.createElement('meter'));
final Tag nav       = createTag((_) => dom.createElement('nav'));
final Tag object    = createTag((_) => dom.createElement('object'));
final Tag ol        = createTag((_) => dom.createElement('ol'));
final Tag optgroup  = createTag((_) => dom.createElement('optgroup'));
final Tag option    = createTag((_) => dom.createElement('option'));
final Tag output    = createTag((_) => dom.createElement('output'));
final Tag p         = createTag((_) => dom.createElement('p'));
final Tag param     = createTag((_) => dom.createElement('param'));
final Tag pre       = createTag((_) => dom.createElement('pre'));
final Tag progress  = createTag((_) => dom.createElement('progress'));
final Tag script    = createTag((_) => dom.createElement('script'));
final Tag select    = createTag((_) => dom.createElement('select'));
final Tag section   = createTag((_) => dom.createElement('section'));
final Tag source    = createTag((_) => dom.createElement('source'));
final Tag span      = createTag((_) => dom.createElement('span'));
final Tag style     = createTag((_) => dom.createElement('style'));
final Tag table     = createTag((_) => dom.createElement('table'));
final Tag tbody     = createTag((_) => dom.createElement('tbody'));
final Tag td        = createTag((_) => dom.createElement('td'));
final Tag template  = createTag((_) => dom.createElement('template'));
final Tag textarea  = createTag((_) => dom.createElement('textarea'));
final Tag tfoot     = createTag((_) => dom.createElement('tfoot'));
final Tag th        = createTag((_) => dom.createElement('th'));
final Tag thead     = createTag((_) => dom.createElement('thead'));
final Tag title     = createTag((_) => dom.createElement('title'));
final Tag toggleButton = createTag((_) => 
    dom.createElement('button', 'toggle-button'));
final Tag toggleOpen = createTag((_) => dom.createElement('toggle-open'));
final Tag tr        = createTag((_) => dom.createElement('tr'));
final Tag track     = createTag((_) => dom.createElement('track'));
final Tag u         = createTag((_) => dom.createElement('u'));
final Tag ul        = createTag((_) => dom.createElement('ul'));
final Tag video     = createTag((_) => dom.createElement('video'));
