part of tags.html;

class CommandBinding extends HtmlElement {
  
  static const TAG = 'command-binding';
  
  // A list of routed commands that have been bound during a single focus event
  static List<RoutedCommand> _bindings = new List<RoutedCommand>();
  
  static EventTarget _focusRoot;
  /// The root in the dom above which there are no more command bindings.
  /// 
  /// By default this is set to the `window` object from `dart:html`.
  static EventTarget get focusRoot => _focusRoot;
  static set focusRoot(EventTarget e) {
    if (_focusRoot != null) {
      _focusRoot.removeEventListener('DOMFocusIn', _onRootFocusIn);
    }
    _focusRoot = e;
    if (_focusRoot != null) {
      _focusRoot.addEventListener('DOMFocusIn', _onRootFocusIn);
    }
  }
  
  static void _onRootFocusIn(focusEvent) => _bindings.clear();
  
  /// Whether or not all command bindings are enabled in the current isolate.
  /// 
  /// By default this is set to `true`; setting to `false` disables _all_ of the
  /// command bindings in the current isolate.
  static bool enableAll = true;
  
  Command _command;
  RoutedCommand _routedCommand;
  
  factory CommandBinding(RoutedCommand routedCommand, Command command) {
    CommandBinding view = new Element.tag(TAG);
    view._routedCommand = routedCommand;
    view._command = command;
    view.attributes['name'] = routedCommand.name;
    return view;
  }

  CommandBinding.created() : super.created();

  void attached() {
    super.attached();
    parent.addEventListener('DOMFocusIn', _onFocusIn);
    parent.addEventListener('keydown', _onKeyDown);
  }
  
  void detached() {
    super.detached();
    parent.removeEventListener('DOMFocusIn', _onFocusIn);
    parent.removeEventListener('keydown', _onKeyDown);
    release();
  }
  
  void release() {
    if (_routedCommand.target == _command) {
      _routedCommand.target = null;
    }
  }
  
  void _onFocusIn(focusEvent) {
    // Check if any element below us in the dom has already made a binding.
    if (!_bindings.contains(_routedCommand)) {
      _routedCommand.target = _command;
      _bindings.add(_routedCommand);
    }
  }
  
  void _onKeyDown(keyEvent) {
    if (enableAll && 
        _routedCommand.target == _command && 
        _routedCommand.matchKeys(keyEvent)) {
      if (_routedCommand.canExecute) {
        keyEvent.preventDefault();
        _routedCommand();
      }
    }
  }
}
